//======================================================== IBJrnd.h
//
// Random Number Generator
// =======================
//
// Copyright (C) Dr. Lutz Janicke, Dunum, Germany, 1999
// email: lj@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// history:
//
// last change: 99-05-03  lj
//
//==================================================================

#include <math.h>

#include "IBJrnd.h"

#define M0       714025
#define IA0        1366
#define IC0      150889
#define RM0      (1.0/M0)
#define NUMSHUF  97
#define RND      (Seed = (IA0*Seed + IC0) % M0)


static int Im0 = 1000000000;
static int Ia0 = 161803398;
static double Rm = 1.e-9;

#define FM0   714025.0

static long Seed = -12345;
static int NumShuf = NUMSHUF;
static char *eMODn = "IBJrnd";

#define JA0    16807
#define JM0    2147483647
#define RJM0   (1./JM0)
#define JQ0    127773
#define JR0    2836
#define JVN    32
#define JVNDIV (1+(JM0-1)/JVN)
#define JEPS   1.2e-7
#define RMAX   (1.0-JEPS)


//=========================================================== RndEqu01
float RndEqu01_1(       // equal distribution between 0 and 1
void )
{
  static int ready=0;
  static int ma[JVN];
  int j, k;
  float temp;

  if (Seed<0 || ready==0) {
    if (-(Seed) <= 1) Seed = 1;
    else Seed = -(Seed);
    for (j=JVN+7; j>=0; j--) {
      k = Seed/JQ0;
      Seed = JA0 * (Seed - k*JQ0) - JR0*k;
      if (Seed < 0) Seed += JM0;
      if (j < JVN) ma[j] = Seed;
      }
    ready = ma[0];
    }
  k = Seed/JQ0;
  Seed = JA0 * (Seed - k*JQ0) - JR0*k;
  if (Seed < 0) Seed += JM0;
  j = ready/JVNDIV;
  ready = ma[j];
  ma[j] = Seed;
  if ((temp = RJM0*ready) > RMAX) return RMAX;
  else return temp;
  }

/*==================================================================== RndSeed
*/
long RndSeed(         /* return current seed  */
void )
  {
  return Seed; }

/*=================================================================== RndEqu01
*/
float RndEqu01_orgold(       /* equal distribution between 0 and 1  */
void )
  {
  static float y;
  float z;
  static int iff=0;
  static int v[NUMSHUF+1];
  int j;
  if (Seed<0 || iff==0) {   /* initialize shuffle table */
    iff=1;
    if (Seed < 0)  Seed = -Seed;
    for (j=1; j<=NumShuf; j++)  v[j] = RND;
    y = RND;
    }
  j = 1 + NumShuf*y/M0;
  y = v[j];
  v[j] = RND;
  z = y/M0;
  return z;
  }



//=========================================================== RndEqu01
float RndEqu01(       // equal distribution between 0 and 1
void )
{
  static int ready=0;
  static int inext, inextp;
  static int ma[56];
  int mj, mk, i, ii, l;

  if (Seed<0 || ready==0) {
    ready = 1;
    mj = Ia0 - ((Seed < 0) ? -Seed : Seed);
    mj %= Im0;
    ma[55] = mj;
    mk = 1;
    for (i=1; i<=54; i++) {
      ii = (21*i) % 55;
      ma[ii] = mk;
      mk = mj - mk;
      if (mk < 0)  mk += Im0;
      mj = ma[ii];
    }
    for (l=1; l<=4; l++)
      for (i=1; i<=55; i++) {
        ma[i] -= ma[1+(i+30)%55];
        if (ma[i] < 0)  ma[i] += Im0;
      }
    inext = 0;
    inextp = 31;
    Seed = 1;
  }
  if (++inext  == 56)  inext = 1;
  if (++inextp == 56)  inextp = 1;
  mj = ma[inext] - ma[inextp];
  if (mj < 0)  mj += Im0;
  ma[inext] = mj;
  return mj*Rm;
}


/*===================================================================== RndEqu
*/
float RndEqu(         /* equal distribution, variance = 1 */
void )
  {
  float r;
  r = 3.4641*(RndEqu01() - 0.5);
  return r;
  }

/*===================================================================== RndNrm
*/
float RndNrm(         /* normal distribution ( average 0,  variance 1 ).  */
void )
  {
  static int iset = 0;
  static float gset;
  float fac, r, v1, v2;

  if (iset == 0) {
    do {
      r = RndEqu01();
      v1 = 2.0*r - 1.0;
      r = RndEqu01();
      v2 = 2.0*r - 1.0;
      r = v1*v1 + v2*v2;
      } while (r >= 1.0);
    fac = sqrt( -2.0*log(r)/r );
    gset = v1*fac;
    iset = 1;
    r = v2*fac;
    return r;
    }
  else {
    iset = 0;
    return gset;
    }
  }

/*===================================================================== RndSet
*/
void RndSet(          /* set new seed           */
long newseed )         /* seed parameter         */
  {
  if (newseed > 0)  Seed = -(newseed % M0);
  else              Seed = -((-newseed) % M0);
  }

/*===========================================================================*/
