//======================================================== IBJtxt.c
//
// Dynamically allocated strings for IBJ-programs
// ==============================================
//
// Copyright (C) Dr. Lutz Janicke, Dunum, Germany, 1999-2000
// email: lj@janicke.de
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License as
// published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// history:
//
// last change: 98-07-31  lj
// 2000-07-27 uj  extended for IBJodor
//
//==================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "IBJtxt.h"
#include "IBJmsg.h"

char *TxtVersion = "1.0.a";

//=============================================================== TxtUnquote
char *TxtUnquote( // unquote a string if necessary
char* s )        // the string
{
  char *p;
  int i, l, start;
  l = strlen(s); if (s[l-1] == '\"') l--;
  start = (s[0] == '\"') ? 1 : 0;
  p = s;
  for (i=start; i<l; i++) *p++ = s[i];
  *p = 0;
  return s;
}

//================================================================= TxtQuote
char *TxtQuote(   // quote a string if necessary
char* s )        // the string
{
  char ss[512], *p;
  int i, l;
  l = strlen(s);
  if (s[0] == '\"') p = ss;
  else { ss[0] = '\"'; p = ss+1; }
  for (i=0; i<l; i++) *p++ = s[i];
  if (s[l-1] != '\"') *p++ = '\"';
  *p = 0;
  return ss;
}

//=================================================================== TxtAlloc
char *TxtAlloc( // allocate string space
int n )     // number of bytes
{
  char *s;
  s = ALLOC(n);
  if (!s) {
    fprintf(stderr, "TXT: allocation of %d bytes failed!\n");
    exit(2);
  }
  memset(s, 0, n);
  return s;
}

//==================================================================== TxtCat
int TxtCat(   // concatenate string to text
TXTSTR *pa,   // text (first part)
char *b )   // string (second part)
  {
  int lb;
  TXTSTR c;
  if (!pa || !b)  return 0;
  lb = strlen(b);
  c.l = pa->l + lb;
  c.s = TxtAlloc(c.l+1);
  if (pa->s) { strcpy(c.s, pa->s);  FREE(pa->s);  pa->s = NULL;  pa->l = 0; }
  strcat(c.s, b);
  *pa = c;
  return 0;
  }

//=================================================================== TxtNCat
int TxtNCat(  // concatenate at most n bytes of string to text
TXTSTR *pa,   // text (first part)
char *b,    // string (second part)
int n )     // number of bytes to add
  {
  int lb;
  TXTSTR c;
  if (!pa || !b)  return 0;
  lb = strlen(b);
  if (n > lb)  n = lb;
  c.l = pa->l + n;
  c.s = TxtAlloc(c.l+1);
  if (pa->s) { strcpy(c.s, pa->s);  FREE(pa->s);  pa->s = NULL;  pa->l = 0; }
  strncat(c.s, b, n);
  c.s[c.l] = 0;
  *pa = c;
  return 0;
  }

//======================================================================== TxtCpy
int TxtCpy(   // copy string to text
TXTSTR *pa,   // text (set)
char *b )   // string (copied)
  {
  int lb;
  if (!pa)  return 0;
  if (!b) {
    pa->s = NULL;
    pa->l = 0;
    return 0;
    }
  lb = strlen(b);
  if (pa->s) { FREE(pa->s);  pa->s = NULL;  pa->l = 0; }
  pa->s = TxtAlloc(lb+1);
  strcpy(pa->s, b);
  pa->l = lb;
  return 0;
  }

//======================================================================== TxtNCpy
int TxtNCpy(  // copy at most n bytes of string to text
TXTSTR *pa,   // text (set)
char *b,    // string (copied)
int n )     // number of bytes
  {
  int lb;
  if (!pa || !b)  return 0;
  lb = strlen(b);
  if (pa->s) { FREE(pa->s);  pa->s = NULL;  pa->l = 0; }
  pa->s = TxtAlloc(n+1);
  strncpy(pa->s, b, n);
  pa->l = n;
  return 0;
  }

//========================================================================== TxtClr
int TxtClr(   // clear text space
TXTSTR *pa )  // text
  {
  if (!pa)  return 0;
  if (pa->s)  FREE(pa->s);
  pa->s = NULL;
  pa->l = 0;
  return 0;
  }

//========================================================================= TxtPCpy
int TxtPCpy(  // copy string to text at position pos
TXTSTR *pa,   // text (set)
int pos,    // position within text
char *b )   // string to be copied
  {
  int lb;
  TXTSTR c;
  if (!pa || !b)  return 0;
  if (pos < 0)  pos = 0;
  if (pos > pa->l)  pos = pa->l;
  lb = strlen(b);
  c.l = pos + lb;
  c.s = TxtAlloc(c.l+1);
  if (pos > 0)  strncpy(c.s, pa->s, pos);
  strcpy(c.s+pos, b);
  if (pa->s)  FREE(pa->s);
  *pa = c;
if (c.l != strlen(c.s)) printf("inconsistent string length!\n");
  return 0;
  }

//========================================================================= TxtPIns
int TxtPIns(  // insert string in text at position pos
TXTSTR *pa,   // text (set)
int pos,    // position within text
char *b )   // string to be inserted
  {
  int lb;
  TXTSTR c;
  if (!pa || !b)  return 0;
  if (pos < 0)  pos = 0;
  if (pos > pa->l)  pos = pa->l;
  lb = strlen(b);
  c.l = pa->l + lb;
  c.s = TxtAlloc(c.l+1);
  if (pos > 0)  strncpy(c.s, pa->s, pos);
  strcpy(c.s+pos, b);
  if (pa->s) {
      strcat(c.s, pa->s+pos);
      FREE(pa->s);
      }
  *pa = c;
if (c.l != strlen(c.s)) printf("inconsistent string length!\n");
  return 0;
  }

//======================================================================= TxtPNDel
int TxtPNdel(   // delete n bytes at position pos
TXTSTR *pa,   // text
int pos,    // position within text
int n )     // number of bytes to delete
  {
  TXTSTR c;
  if (!pa)  return 0;
  if (pos < 0)  pos = 0;
  if (pos > pa->l)  pos = pa->l;
  if (n < 0)  n = 0;
  if (pos+n > pa->l)  c.l = pos;
  else                c.l = pa->l - n;
  c.s = TxtAlloc(c.l+1);
  if (pos > 0)  strncpy(c.s, pa->s, pos);
  if (pos >= c.l)  c.s[c.l] = 0;
  if (pa->s) {
      strcpy(c.s+pos, pa->s+pos+n);
      FREE(pa->s);
      }
  *pa = c;
if (c.l != strlen(c.s)) printf("inconsistent string length!\n");
  return 0;
  }

//========================================================================= TxtRpl
int TxtRpl(   // replace string within text
TXTSTR *pa,   // text
char *old,    // string pattern to be replaced
char *new )   // replacing string
  {
  int lold, lnew, n;
  char *p, *q;
  TXTSTR c;
  if (!pa || !old || !new)  return 0;
  lold = strlen(old);
  lnew = strlen(new);
  if (lold == 0)  return 0;
  if (!pa->l)  return 0;
  if (lold == lnew) {
      p = pa->s;
      while (NULL != (p = strstr(p, old))) {
          strncpy(p, new, lnew);
          p += lnew; }
      return 0; }
  n = 0;
  p = pa->s;
  while (NULL != (p = strstr(p, old))) {
      n++;
      p += lold; }
  if (n == 0)  return 0;
  c.l = pa->l + n*(lnew-lold);
  c.s = TxtAlloc(c.l+1);
  p = pa->s;
  while (NULL != (q = strstr(p, old))) {
      strncat(c.s, p, q-p);
      strcat(c.s, new);
      p = q + lold; }
  strcat(c.s, p);
  FREE(pa->s);
  *pa = c;
if (c.l != strlen(c.s)) printf("inconsistent string length!\n");
  return 0;
  }

#ifdef  MAIN  //###################################################################

int main( int argc, char *argv() )
  {
  TXTSTR t = { NULL, 0 };
  printf("Test auf TXT-Funktionen\n");
  TxtCpy(&t, "abcd");
  TxtCat(&t, "1234");
  printf("cat=%s\n", t.s);
  TxtClr(&t);
  TxtCpy(&t, "12ab34ab56");
  TxtRpl(&t, "a", "xy");
  printf("rpl=%s\n", t.s);
  return 0;
  }

#endif //##########################################################################
